@@ -1,5 +1,4 @@
-/*  $Id: chan.c 9801 2015-03-21 13:37:54Z iulius $
-**
+/*
 **  I/O channel (and buffer) processing.
 **
 **  This file is the heart of innd.  Everything innd does is represented by
@@ -14,14 +13,14 @@
 **  to whatever channels have work to do.
 */
 
-#include "config.h"
-#include "clibrary.h"
+#include "portable/system.h"
 
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #ifdef HAVE_SYS_SELECT_H
-# include <sys/select.h>
+#    include <sys/select.h>
 #endif
 
+#include "inn/fdflag.h"
 #include "inn/innconf.h"
 #include "inn/network.h"
 #include "innd.h"
@@ -31,32 +30,26 @@
    to 0 if they don't exist so that we can unconditionally compare errno to
    them in the code. */
 #ifndef ENOTSOCK
-# define ENOTSOCK 0
+#    define ENOTSOCK 0
 #endif
 #ifndef ENOTTY
-# define ENOTTY 0
+#    define ENOTTY 0
 #endif
 
-static const char * const timer_name[] = {
-    "idle", "artclean", "artwrite", "artcncl", "sitesend", "overv",
-    "perl", "python", "nntpread", "artparse", "artlog", "datamove"
-};
-
-/* Compaction threshold as a divisor of the buffer size.  If the amount free
-   at the beginning of the buffer is bigger than the quotient, it is compacted
-   in the read loop. */
-#define COMP_THRESHOLD 10
+static const char *const timer_name[] = {
+    "idle", "artclean", "artwrite", "artcncl",  "sitesend", "overv",
+    "perl", "python",   "nntpread", "artparse", "artlog",   "datamove"};
 
 /* Global data about the channels. */
 struct channels {
     fd_set read_set;
     fd_set sleep_set;
     fd_set write_set;
-    int sleep_count;            /* Number of sleeping channels. */
-    int max_fd;                 /* Max fd in read_set or write_set. */
-    int max_sleep_fd;           /* Max fd in sleep_set. */
-    int table_size;             /* Total number of channels. */
-    CHANNEL *table;             /* Table of channel structs. */
+    int sleep_count;  /* Number of sleeping channels. */
+    int max_fd;       /* Max fd in read_set or write_set. */
+    int max_sleep_fd; /* Max fd in sleep_set. */
+    int table_size;   /* Total number of channels. */
+    CHANNEL *table;   /* Table of channel structs. */
 
     /* Special prioritized channels, for the control and remconn channels.  We
        check these first each time. */
@@ -76,6 +69,29 @@ static struct channels channels;
 static CHANNEL CHANnull;
 
 
+/*
+**  Returns true if socket activation is used and the channel given as
+**  as argument is used for socket activation.  Returns false otherwise.
+*/
+bool
+CHANsystemdsa(CHANNEL *cp)
+{
+    const char *s;
+    int count;
+
+    s = getenv("INN_SD_LISTEN_FDS_COUNT");
+    if (s == NULL)
+        return false;
+
+    count = atoi(s);
+
+    if (cp->fd >= SD_LISTEN_FDS_START && cp->fd < SD_LISTEN_FDS_START + count)
+        return true;
+
+    return false;
+}
+
+
 /*
 **  Tear down our world.  Free all of the allocated channels and clear all
 **  global state data.  This function can also be used to initialize the
@@ -97,6 +113,8 @@ CHANshutdown(void)
     if (channels.table != NULL) {
         cp = channels.table;
         for (i = channels.table_size; --i >= 0; cp++) {
+            if (CHANsystemdsa(cp))
+                continue;
             if (cp->Type != CTfree)
                 CHANclose(cp, CHANname(cp));
             if (cp->In.data)
@@ -164,8 +182,8 @@ CHANcreate(int fd, enum channel_type type, enum channel_state state,
 {
     CHANNEL *cp;
     int i, j, size;
-    struct buffer in  = { 0, 0, 0, NULL };
-    struct buffer out = { 0, 0, 0, NULL };
+    struct buffer in = {0, 0, 0, NULL};
+    struct buffer out = {0, 0, 0, NULL};
 
     /* Currently, we do no dynamic allocation, but instead assume that the
        channel table is sized large enough to hold all possible file
@@ -203,14 +221,14 @@ CHANcreate(int fd, enum channel_type type, enum channel_state state,
     HashClear(&cp->CurrentMessageIDHash);
     ARTprepare(cp);
 
-    close_on_exec(fd, true);
+    fdflag_close_exec(fd, true);
 
 #ifndef _HPUX_SOURCE
     /* Stupid HPUX 11.00 has a broken listen/accept where setting the listen
        socket to nonblocking prevents you from successfully setting the
        socket returned by accept(2) back to blocking mode, no matter what,
-       resulting in all kinds of funny behaviour, data loss, etc. etc.  */
-    if (nonblocking(fd, true) < 0 && errno != ENOTSOCK && errno != ENOTTY)
+       resulting in all kinds of funny behaviour, data loss, etc. etc. */
+    if (!fdflag_nonblocking(fd, true) && errno != ENOTSOCK && errno != ENOTTY)
         syswarn("%s cant nonblock %d", LogName, fd);
 #endif
 
@@ -221,8 +239,8 @@ CHANcreate(int fd, enum channel_type type, enum channel_state state,
                 break;
         if (i >= channels.prioritized_size) {
             size = channels.prioritized_size + 1;
-            channels.prioritized
-                = xrealloc(channels.prioritized, size * sizeof(CHANNEL *));
+            channels.prioritized =
+                xrealloc(channels.prioritized, size * sizeof(CHANNEL *));
             for (j = channels.prioritized_size; j < size; j++)
                 channels.prioritized[j] = NULL;
             channels.prioritized_size = size;
@@ -253,8 +271,8 @@ CHANtracing(CHANNEL *cp, bool flag)
                cp->BadWrites, cp->BlockedWrites, cp->BadReads);
         network_sockaddr_sprint(addr, sizeof(addr),
                                 (struct sockaddr *) &cp->Address);
-        notice("%s trace address %s lastactive %ld nextlog %ld", name,
-               addr, (long) cp->LastActive, (long) cp->NextLog);
+        notice("%s trace address %s lastactive %ld nextlog %ld", name, addr,
+               (long) cp->LastActive, (long) cp->NextLog);
         if (FD_ISSET(cp->fd, &channels.sleep_set))
             notice("%s trace sleeping %ld 0x%p", name, (long) cp->Waketime,
                    (void *) cp->Waker);
@@ -285,24 +303,26 @@ CHANclose_nntp(CHANNEL *cp, const char *name)
     NCclearwip(cp);
     if (cp->State == CScancel)
         notice("%s closed seconds %ld cancels %ld", name,
-               (long)(Now.tv_sec - cp->Started), cp->Received);
+               (long) (Now.tv_sec - cp->Started), cp->Received);
     else {
-        notice("%s checkpoint seconds %ld accepted %ld refused %ld rejected %ld"
-               " duplicate %ld accepted size %.0f duplicate size %.0f"
-               " rejected size %.0f", name,
-               (long)(Now.tv_sec - cp->Started_checkpoint),
-               cp->Received - cp->Received_checkpoint,
-               cp->Refused - cp->Refused_checkpoint,
-               cp->Rejected - cp->Rejected_checkpoint,
-               cp->Duplicate - cp->Duplicate_checkpoint,
-               cp->Size - cp->Size_checkpoint,
-               cp->DuplicateSize - cp->DuplicateSize_checkpoint,
-               cp->RejectSize - cp->RejectSize_checkpoint);
+        notice(
+            "%s checkpoint seconds %ld accepted %ld refused %ld rejected %ld"
+            " duplicate %ld accepted size %.0f duplicate size %.0f"
+            " rejected size %.0f",
+            name, (long) (Now.tv_sec - cp->Started_checkpoint),
+            cp->Received - cp->Received_checkpoint,
+            cp->Refused - cp->Refused_checkpoint,
+            cp->Rejected - cp->Rejected_checkpoint,
+            cp->Duplicate - cp->Duplicate_checkpoint,
+            (double) (cp->Size - cp->Size_checkpoint),
+            (double) (cp->DuplicateSize - cp->DuplicateSize_checkpoint),
+            (double) (cp->RejectSize - cp->RejectSize_checkpoint));
         notice("%s closed seconds %ld accepted %ld refused %ld rejected %ld"
                " duplicate %ld accepted size %.0f duplicate size %.0f"
-               " rejected size %.0f", name, (long)(Now.tv_sec - cp->Started),
-               cp->Received, cp->Refused, cp->Rejected, cp->Duplicate,
-               cp->Size, cp->DuplicateSize, cp->RejectSize);
+               " rejected size %.0f",
+               name, (long) (Now.tv_sec - cp->Started), cp->Received,
+               cp->Refused, cp->Rejected, cp->Duplicate, (double) cp->Size,
+               (double) cp->DuplicateSize, (double) cp->RejectSize);
     }
     if (cp->Data.Newsgroups.Data != NULL) {
         free(cp->Data.Newsgroups.Data);
@@ -388,7 +408,8 @@ CHANclose(CHANNEL *cp, const char *name)
         if (cp->Type == CTnntp)
             CHANclose_nntp(cp, name);
         else if (cp->Type == CTreject)
-            notice("%s %ld", name, cp->Rejected); /* Use cp->Rejected for the response code. */
+            notice("%s %ld", name,
+                   cp->Rejected); /* Use cp->Rejected for the response code. */
         else if (cp->Out.left)
             warn("%s closed lost %lu", name, (unsigned long) cp->Out.left);
         else
@@ -463,8 +484,8 @@ CHANname(CHANNEL *cp)
 
     switch (cp->Type) {
     default:
-        snprintf(cp->Name, sizeof(cp->Name), "?%d(#%d@%ld)?", cp->Type,
-                 cp->fd, (long) (cp - channels.table));
+        snprintf(cp->Name, sizeof(cp->Name), "?%d(#%d@%ld)?", cp->Type, cp->fd,
+                 (long) (cp - channels.table));
         break;
     case CTany:
         snprintf(cp->Name, sizeof(cp->Name), "any:%d", cp->fd);
@@ -539,7 +560,7 @@ CHANNEL *
 CHANiter(int *ip, enum channel_type type)
 {
     CHANNEL *cp;
-    int i = *ip;
+    int i;
 
     for (i = *ip; i >= 0 && i < channels.table_size; i++) {
         cp = &channels.table[i];
@@ -563,7 +584,7 @@ static void
 CHANresetlast(int fd)
 {
     if (fd == channels.max_fd)
-        while (   !FD_ISSET(channels.max_fd, &channels.read_set)
+        while (!FD_ISSET(channels.max_fd, &channels.read_set)
                && !FD_ISSET(channels.max_fd, &channels.write_set)
                && channels.max_fd > 1)
             channels.max_fd--;
@@ -578,7 +599,7 @@ static void
 CHANresetlastsleeping(int fd)
 {
     if (fd == channels.max_sleep_fd) {
-        while (   !FD_ISSET(channels.max_sleep_fd, &channels.sleep_set)
+        while (!FD_ISSET(channels.max_sleep_fd, &channels.sleep_set)
                && channels.max_sleep_fd > 1)
             channels.max_sleep_fd--;
     }
@@ -761,8 +782,8 @@ CHANresize(CHANNEL *cp, size_t size)
     bp->data = xrealloc(bp->data, bp->size);
     offset = p - bp->data;
     if (offset != 0) {
-        if (cp->State == CSgetheader || cp->State == CSgetbody ||
-            cp->State == CSeatarticle) {
+        if (cp->State == CSgetheader || cp->State == CSgetbody
+            || cp->State == CSeatarticle) {
             if (cp->Data.BytesHeader != NULL)
                 cp->Data.BytesHeader -= offset;
             for (i = 0; i < MAX_ARTHEADER; i++, hc++) {
@@ -789,7 +810,7 @@ CHANreadtext(CHANNEL *cp)
     /* Grow buffer if we're getting close to current limit.
 
        FIXME: The In buffer doesn't use the normal meanings of .used and
-       .left.  */
+       .left. */
     bp = &cp->In;
     bp->left = bp->size - bp->used;
     if (bp->left <= LOW_WATER)
@@ -1007,7 +1028,7 @@ CHANdiagnose(void)
 void
 CHANcount_active(CHANNEL *cp)
 {
-    int found;  
+    int found;
     CHANNEL *tempchan;
     char *label, *tmplabel;
     int fd;
@@ -1172,8 +1193,13 @@ CHANreadloop(void)
                 tv.tv_sec = innconf->timer;
             }
         }
+
+        /* Mask signals when not in select to prevent a signal handler
+           from accessing data that the main code is mutating. */
         TMRstart(TMR_IDLE);
+        xsignal_unmask();
         count = select(channels.max_fd + 1, &rdfds, &wrfds, NULL, &tv);
+        xsignal_mask();
         TMRstop(TMR_IDLE);
 
         if (count < 0) {
@@ -1181,14 +1207,20 @@ CHANreadloop(void)
                 syswarn("%s cant select", LogName);
 #ifdef INND_FIND_BAD_FDS
                 CHANdiagnose();
-#endif      
+#endif
             }
             continue;
         }
 
         STATUSmainloophook();
         if (GotTerminate) {
-            warn("%s exiting due to signal", LogName);
+#ifdef DO_PERL
+            PLmode(Mode, OMshutdown, (char *) "exiting due to signal");
+#endif
+#ifdef DO_PYTHON
+            PYmode(Mode, OMshutdown, (char *) "exiting due to signal");
+#endif
+            notice("%s exiting due to signal", LogName);
             CleanupAndExit(0, NULL);
         }
 
@@ -1260,7 +1292,7 @@ CHANreadloop(void)
                     continue;
                 }
             }
-            
+
             /* Anything to read? */
             if (FD_ISSET(fd, &channels.read_set) && FD_ISSET(fd, &rdfds)) {
                 count--;
