@@ -22,7 +22,7 @@
 
 #include "config.h"
 #include "portable/macros.h"
-#include "clibrary.h"
+#include "portable/system.h"
 
 #include <errno.h>
 
