@@ -1,15 +1,14 @@
-/* $Id: pread-t.c 5379 2002-03-31 21:45:12Z rra $ */
 /* pread test suite. */
 
-#include "config.h"
-#include "clibrary.h"
+#include "portable/system.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 
+#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "libtest.h"
+#include "tap/basic.h"
 
 ssize_t test_pread(int fd, void *buf, size_t nbyte, off_t offset);
 
@@ -35,7 +34,7 @@ main(void)
         sysdie("Can't rewind .testout");
     memset(result, 0, sizeof(result));
 
-    puts("6");
+    test_init(6);
 
     status = test_pread(fd, result, 128, 128);
     ok(1, (status == 128) && !memcmp(result, buf + 128, 128));
